%
%  Quick and dirty code to make stacks ala the IRIS globalstacks
%  https://ds.iris.edu/ds/products/globalstacks/
%  Alex Hutko, IRIS
%  IRIS-EarthScope USArray Data Processing and Analysis Short Course, 2015
%

clear all;
tic

%---------- set parameters

dt = 0.5                    %--- sample rate of data
STAlen = 5.0/dt;            %--- short term average window lenth in Sec
LTAlen = STAlen*10;
Hanntaperlen = 10./dt;      %--- width of smoothing cos^2 taper
STALTAsaturation = 10;      %--- saturation level of STA/LTA functions
Npower = 2;                 %--- power of Nth root stacking

Pwindowlen = 30*60/dt;      %--- try to make this before the surface waves
prePwindowlen = 10*60/dt;   %--- make sure this is before first P arrival

n1=1                       %--- index of the first trace to plot
ntracesplot=10;             %--- nubmer of traces to plot

%---------- read raw sac files with consistent sampling rate & length

fileID = fopen('saclist');
C = textscan(fileID,'%s');
nSAC = length(C{1});
%nSAC=100;
for i = 1:nSAC
   fprintf('Reading %d of %d\n', i, nSAC); 
   sacfile = C{1}{i};
   A = rdSac(sacfile);
   if ( i == 1 ) 
      tracelen=length(A);
   end
   dataraw(i,1:tracelen) = A(1:tracelen)./max(abs(A(1:tracelen)));
end

%---------- make the taper

clear taper
for i=1:Hanntaperlen+1
   taper(i) = 1-(cos((i-1)*pi/Hanntaperlen)^2);
end

%---------- process the traces

normfactor=LTAlen/STAlen;
envelope = zeros(nSAC,tracelen);
STALTA = zeros(nSAC,tracelen);

for i=1:nSAC;
   %----- envelope & smooth trace w taper
   fprintf('Smoothing and making STA/LTA %d of %d\n', i, nSAC);                                       
   temp=conv(abs(dataraw(i,:)),taper);
   envelope(i,:)=temp(1:tracelen);

   %----- normalize on the first X minutes
   peakamp = max(abs(envelope(i,1:Pwindowlen)));
   envelope(i,:) = envelope(i,:)./peakamp;

   %----- subtract pre-P median noise
   noise = max(envelope(i,1:prePwindowlen));
   envelope(i,:) = envelope(i,:) - noise;

   %----- add a water level
   envelope(i,(envelope(i,:)<0.001))=0.001;

   %----- make simple STA/LTA function of envelope
   LTAsum=sum(envelope(i,1:LTAlen));
   STAsum=sum(envelope(i,LTAlen+1:LTAlen+1+STAlen));
   for j = LTAlen+1:tracelen-STAlen-1
      STALTA(i,j)=normfactor*STAsum/LTAsum;
      LTAsum = LTAsum - envelope(i,j-LTAlen) + envelope(i,j);
      STAsum = STAsum - envelope(i,j) + envelope(i,j+STAlen);
   end

   %----- saturate the STA/LTA function
   STALTA(i,(STALTA(i,:) > STALTAsaturation)) = STALTAsaturation;

   %----- weigh the STALTA by pre-P noise
   STALTA(i,:)=STALTA(i,:)*(1-noise);

end

dataraw(isnan(dataraw))=0;
envelope(isnan(envelope))=0;
STALTA(isnan(STALTA))=0;

%---------- stack

datataperwidth = round(0.01*tracelen);
Nroot = 1/Npower;
stackdataraw=zeros(tracelen,1);
stackenvelope=zeros(tracelen,1);
stackSTALTA =zeros(tracelen,1);

for i=1:nSAC
   fprintf('Stacking %d of %d\n', i, nSAC);
   for j= datataperwidth+LTAlen:tracelen-datataperwidth
      stackdataraw(j)=stackdataraw(j)+(abs(dataraw(i,j))^Nroot)*sign(dataraw(i,j));
      stackenvelope(j)=stackenvelope(j)+(abs(envelope(i,j))^Nroot)*sign(envelope(i,j));
      stackSTALTA(j)= stackSTALTA(j)+(abs(STALTA(i,j))^Nroot)*sign(STALTA(i,j));
   end
end

stackdataraw=stackdataraw.^Npower;
stackenvelope=stackenvelope.^Npower;
stackSTALTA = stackSTALTA.^Npower;

trimlen = datataperwidth + LTAlen;
stackenvelope(1:trimlen)=NaN;
stackenvelope(tracelen-trimlen:tracelen)=NaN;
stackSTALTA(1:trimlen)=NaN;
stackSTALTA(tracelen-trimlen:tracelen)=NaN;

%---------- plot

time=(1:tracelen).*dt/60;
n2 = n1 + ntracesplot;

fig=figure('units','pixels','outerposition',[50 50 700 300 ],'paperpositionmode','auto');  %   left up width height
subplot(1,3,1)
for i=n1:n2
   plot(time,i+dataraw(i,:).*0.8,'k')
   hold on
end
plot(time,i+1+2*stackdataraw/max(abs(stackdataraw)),'r');
title('filtered traces')
xlabel('minutes')
axis('tight')

subplot(1,3,2)
for i=n1:n2
   plot(time,i+0.8* envelope(i,:),'k')
   hold on
end
plot(time,i+1+ 2*stackenvelope/max(abs(stackenvelope)),'r');
title('processed envelopes')
xlabel('minutes')
axis('tight')

subplot(1,3,3)
for i=n1:n2
   plot(time,i+0.8* STALTA(i,:)./STALTAsaturation,'k')
   hold on
end
plot(time,i+1+ 2*stackSTALTA/max(abs(stackSTALTA)),'r');
title('weighted STA/LTA functions')
xlabel('minutes')
axis('tight')
print -depsc tracesandstacks.eps

fig=figure('units','pixels','outerposition',[50 50 700 300 ],'paperpositionmode','auto');  %   left up width height
plot(time,stackSTALTA,'k')
title('STA/LTA stack')
xlabel('minutes')

txt=num2str(nSAC,'Ntraces=%d')   ;  text(max(time)*.9,max(stackSTALTA)*.98,txt)
txt=num2str(STALTAsaturation,'STALTAsaturation=%.1f')   ;  text(max(time)*.9,max(stackSTALTA)*.9,txt)
txt=num2str(STAlen*dt,'STA width=%.1f')   ;  text(max(time)*.9,max(stackSTALTA)*.8,txt)
txt=num2str(LTAlen*dt,'LTA width=%.1f')   ;  text(max(time)*.9,max(stackSTALTA)*.7,txt)
txt=num2str(Hanntaperlen*dt,'Hann width=%.1f')   ;  text(max(time)*.9,max(stackSTALTA)*.6,txt)
txt=num2str(Npower,'Nth root=%.1f')   ;  text(max(time)*.9,max(stackSTALTA)*.5,txt)
txt=num2str(prePwindowlen*dt ,'prePwindow width=%.1f')   ;  text(max(time)*.9,max(stackSTALTA)*.4,txt)

print -depsc stack.eps

toc



